////////////////////////////////////////////////////////////////////////////////
//
//	COMMENTAIRES
  //
  //	Nom					reversi.sce
  //	Auteur				Jérôme LABATUT 
  //	Date de création	2017-01-20
  //
  //	Version Scilab		5.5.2
  //	Module Atoms requis	Aucun
  //
  //	Objectif			Implémentation du jeu Reversi
//
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//
//	INITIALISATION
	
	funcprot(0)
	clearglobal()
	clear()
	
	xdel(winsid())
	tohome()
	clc()
	
	global JEU
//
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//
//	CONSTANTES
//
////////////////////////////////////////////////////////////////////////////////

function affecterParametres()
	global JEU
	
	// Paramètres du modèle (damier et joueurs)
		JEU.DAMIER.DIRECTIONS_NOMBRE = 8
		JEU.DAMIER.DIRECTIONS_ANGLES = ((1:JEU.DAMIER.DIRECTIONS_NOMBRE) - 1)'/JEU.DAMIER.DIRECTIONS_NOMBRE*2*%pi
		JEU.DAMIER.DIRECTIONS = round([cos(JEU.DAMIER.DIRECTIONS_ANGLES), sin(JEU.DAMIER.DIRECTIONS_ANGLES)])
		
		JEU.DAMIER.IMPOSSIBLE = [0, 0];
		
		JEU.ALGORITHME.Humain = 1;
		JEU.ALGORITHME.Robot =  2; // Choix aléatoire parmi les meilleures cases
		JEU.ALGORITHME.Robot1 = 3; // Choix déterministe parmi les meilleures cases
		
		JEU.ETAT.Inactif = 0;
		JEU.ETAT.Actif = 1;
		JEU.ETAT.Bloque = 2;
		JEU.ETAT.Elimine = 3;
		
		angle = (0:360)'/360*2*%pi;
		xPion = 0.5 + 0.4*cos(angle);
		yPion = 0.5 + 0.4*sin(angle);
		
		Noir.Nom = "Noir";
		Noir.Code = 1;
		Noir.Algorithme = JEU.ALGORITHME.Humain;
		Noir.Etat = JEU.ETAT.Actif;
		Noir.X = xPion;
		Noir.Y = yPion;
		Noir.CouleurRVB = [0, 0, 0];
		Noir.CouleurCode = color("black");
		
		Blanc.Nom = "Blanc";
		Blanc.Code = 2;
		Blanc.Algorithme = JEU.ALGORITHME.Robot;
		Blanc.Etat = JEU.ETAT.Actif;
		Blanc.X = xPion;
		Blanc.Y = yPion;
		Blanc.CouleurRVB = [1, 1, 1];
		Blanc.CouleurCode = color("white");
		
		Rouge.Nom = "Rouge";
		Rouge.Code = 3;
		Rouge.Algorithme = JEU.ALGORITHME.Robot;
		Rouge.Etat = JEU.ETAT.Inactif;
		Rouge.X = xPion;
		Rouge.Y = yPion;
		Rouge.CouleurRVB = [1, 0, 0];
		Rouge.CouleurCode = color("red");
		
		Bleu.Nom = "Bleu";
		Bleu.Code = 4;
		Bleu.Algorithme = JEU.ALGORITHME.Robot;
		Bleu.Etat = JEU.ETAT.Inactif;
		Bleu.X = xPion;
		Bleu.Y = yPion;
		Bleu.CouleurRVB = [0, 0, 1];
		Bleu.CouleurCode = color("blue");
		
		Vert.Nom = "Vert";
		Vert.Code = 5;
		Vert.Algorithme = JEU.ALGORITHME.Robot;
		Vert.Etat = JEU.ETAT.Inactif;
		Vert.X = xPion;
		Vert.Y = yPion;
		Vert.CouleurRVB = [0, 1, 0];
		Vert.CouleurCode = color("dark green");
		
		Jaune.Nom = "Jaune";
		Jaune.Code = 6;
		Jaune.Algorithme = JEU.ALGORITHME.Robot;
		Jaune.Etat = JEU.ETAT.Inactif;
		Jaune.X = xPion;
		Jaune.Y = yPion;
		Jaune.CouleurRVB = [0, 1, 1];
		Jaune.CouleurCode = color("yellow");
		
		Violet.Nom = "Violet";
		Violet.Code = 7;
		Violet.Algorithme = JEU.ALGORITHME.Robot;
		Violet.Etat = JEU.ETAT.Inactif;
		Violet.X = xPion;
		Violet.Y = yPion;
		Violet.CouleurRVB = [1, 0, 1];
		Violet.CouleurCode = color("violet");
		
		Orange.Nom = "Orange";
		Orange.Code = 8;
		Orange.Algorithme = JEU.ALGORITHME.Robot;
		Orange.Etat = JEU.ETAT.Inactif;
		Orange.X = xPion;
		Orange.Y = yPion;
		Orange.CouleurRVB = [1, 1, 0];
		Orange.CouleurCode = color("orange");
		
		JEU.JOUEURS = list(Noir, Blanc, Rouge, Bleu, Vert, Jaune, Violet, Orange);
		JEU.JOUEURS_NOMBRE = length(JEU.JOUEURS);
	//
	
	// Paramètres des vues (interface, damier et bandeau)
		JEU.INTERFACE.Nom = "Interface";
		JEU.INTERFACE.Titre = "Reversi";
		JEU.INTERFACE.Position = [0, 0];
		JEU.INTERFACE.Taille = [700, 700];
		JEU.INTERFACE.CouleurRVB = [0.8 0.8 0.8];
		
		JEU.DAMIER.Position = [0, 0, 0.75, 0.75];
		JEU.DAMIER.Marges = [0, 0, 0, 0];
		
		JEU.DAMIER.Selection = 3;		//  3 - Clic gauche
		JEU.DAMIER.Confirmation = 4;	//  4 - Clic centre
		JEU.DAMIER.Annulation = 11;		// 11 - Double-clic centre
		
		JEU.CASE.X = [0; 1; 1; 0];
		JEU.CASE.Y = [0; 0; 1; 1];
		JEU.CASE.CouleurCode = color("green");
		JEU.CASE.CouleurRVB = [0.0 0.7 0.0];
		
		JEU.CASE_JOUABLE.X = [0.4; 0.6; 0.6; 0.4];
		JEU.CASE_JOUABLE.Y = [0.4; 0.4; 0.6; 0.6];
		
		JEU.BANDEAU.Nom = "BandeauInformations";	
		JEU.BANDEAU.Position = [10 10 500 100];
		JEU.BANDEAU.TaillePolice = 12;
		JEU.BANDEAU.CouleurRVB = [0.7 0.7 0.7];
	//
	
	// Parametres des contrôleurs (menus, boutons et partie)
		JEU.MENUS.Nom11 = "SélectionGrille";
		JEU.MENUS.Nom12 = "SélectionTaille";
		JEU.MENUS.Nom13 = "SélectionJoueurHumain";
		JEU.MENUS.Nom13Choix = ["SélectionJoueurHumainNoir";	...
								"SélectionJoueurHumainBlanc";	...
								"SélectionJoueurHumainRouge";	...
								"SélectionJoueurHumainBleu";	...
								"SélectionJoueurHumainViolet";	...
								"SélectionJoueurHumainJaune";	...
								"SélectionJoueurHumainVert";	...
								"SélectionJoueurHumainOrange"];
		
		JEU.BOUTONS.Nom15 = "ControleurPartie";
		JEU.BOUTONS.Nom25 = "ControleurAffichageCasesJouables";
		JEU.BOUTONS.Nom35 = "ControleurDécimation";
		JEU.BOUTONS.Nom45 = "ArretPartie";
		JEU.BOUTONS.Nom55 = "SortieJeu";	
		
		JEU.BOUTONS.Espace = 0.2;
		JEU.BOUTONS.Bordure = 1/10*JEU.BOUTONS.Espace;
		JEU.BOUTONS.Taille = JEU.BOUTONS.Espace - 2*JEU.BOUTONS.Bordure;
		JEU.BOUTONS.Position = [JEU.BOUTONS.Bordure, JEU.BOUTONS.Bordure, JEU.BOUTONS.Taille, JEU.BOUTONS.Taille];
		
		JEU.BOUTONS.Position15 = [4*JEU.BOUTONS.Espace 4*JEU.BOUTONS.Espace 0 0] + JEU.BOUTONS.Position;
		JEU.BOUTONS.Position25 = [4*JEU.BOUTONS.Espace 3*JEU.BOUTONS.Espace 0 0] + JEU.BOUTONS.Position;
		JEU.BOUTONS.Position35 = [4*JEU.BOUTONS.Espace 2*JEU.BOUTONS.Espace 0 0] + JEU.BOUTONS.Position;
		JEU.BOUTONS.Position45 = [4*JEU.BOUTONS.Espace JEU.BOUTONS.Espace   0 0] + JEU.BOUTONS.Position;
		JEU.BOUTONS.Position55 = [4*JEU.BOUTONS.Espace 0 0 0] + JEU.BOUTONS.Position;
		
		JEU.BOUTONS.CouleurRVB = [0.9 0.9 0.9];
		JEU.BOUTONS.TaillePolice = 15;
		
		JEU.ETAT_PARTIE.ACTIVABLE = 10;							// Sélection à faire ou à confirmer
		JEU.ETAT_PARTIE.ACTIVE = 11;							// Partie en cours
		JEU.ETAT_PARTIE.BLOQUEE = 12;							// Partie bloquée : tous les joueurs sont bloqués
		JEU.ETAT_PARTIE.INTERROMPUE = 13;						// Partie interrompue par l'utilisateur
		JEU.ETAT_PARTIE.COMPLETEE = 14;							// Damier complété
		JEU.ETAT_PARTIE.REINITIALISABLE = 15;					// Partie terminée : vainqueur affiché, effacage du damier en attente
		
		JEU.Partie.NumeroGrille = 210;
		JEU.Partie.NombreJoueurs = 2;
		JEU.Partie.Taille = 12;
		JEU.Partie.CodeHumain = 1;
		
		JEU.Partie.Etat = JEU.ETAT_PARTIE.ACTIVABLE;
		
		JEU.Partie.AffichageAide = 0;
		// 0 : pas d'affichage
		// 1 : affichage des coups jouables par le joueur humain
		// 2 : affichage des coups jouables pour chaque joueur
		
		JEU.Partie.Decimation = 0;
		// 0 : pas de sélection
		// 1 : sélection confirmée, à exécuter
		
		JEU.Partie.PAUSE = 100;
	//
endfunction

////////////////////////////////////////////////////////////////////////////////
//
//	FONCTIONS MODELE
  //
  //			Fonction principale du jeu
  //			Pose d'un pion dans une case jouable et retournement des pions 
  //			Enlèvement des pions sélectionnés
  //			Calcul d'une grille de départ de taille donnée
  //
  //	Joueur	Heuristique du robot aléatoire
  //	Joueur	Heuristique du robot déterministe
  //	Joueur	Sélection d'une case par le joueur humain
  //	Joueur	Sélection d'un rectangle de cases à décimer
  //
  //	Arbitre	Calcul des cases jouables (coordonnées, score et "connectivité")
  //	Arbitre	Calcul du score d'une case (0 si elle n'est pas jouable)
  //	Arbitre	Calcul des effectifs par ordre décroissant
//
////////////////////////////////////////////////////////////////////////////////

function GrilleS = jouerPartie(Grille)
	global JEU
	
	JEU.Partie.Etat = JEU.ETAT_PARTIE.ACTIVE;
	JEU.Partie.NombreCoups = JEU.Partie.Taille^2;
	NombreCoups = 0;											// Initialisation du compteur anti-boucle infinie
	
	for Code = 1:JEU.Partie.NombreJoueurs						// Activation de tous les joueurs sélectionnés
		JEU.JOUEURS(Code).Etat = JEU.ETAT.Actif;
	end
	
	//															// Début de la boucle sur les critères d'arrêt de jeu
	while (NombreCoups < JEU.Partie.NombreCoups)&(JEU.Partie.Etat == JEU.ETAT_PARTIE.ACTIVE)
		//														// Début de la boucle sur le nombre de joueurs sélectionnés
		for Code = 1:JEU.Partie.NombreJoueurs
			Etat = JEU.JOUEURS(Code).Etat;						// Etat du joueur courant
			Algorithme = JEU.JOUEURS(Code).Algorithme;			// Algorithme du joueur courant
			
			if (JEU.Partie.Decimation == 1)						// Décimation des pions sélectionnés
				Grille = enleverPions(Grille, JEU.Selection)
				
				controlerDecimationPions()
			end
			
			if (Etat == JEU.ETAT.Actif)							// Affichage des cases jouables pour les joueurs actifs
				if (JEU.Partie.AffichageAide == 1)
					voirCasesJouables(Grille, JEU.Partie.CodeHumain)				
					
					sleep(JEU.Partie.PAUSE)
				end
				if (JEU.Partie.AffichageAide == 2)
					voirCasesJouables(Grille, Code)
					
					sleep(JEU.Partie.PAUSE)
				end
			end
			
			//													// Sélection du coup par le joueur courant s'il est actif ...
			if (Etat == JEU.ETAT.Actif)|(Etat == JEU.ETAT.Bloque)
				select Algorithme
				case JEU.ALGORITHME.Humain						// ... et humain (choix de l'utilisateur)
					Coup = jouerHumain(Grille, Code)
				case JEU.ALGORITHME.Robot						// ... et robot (choix aléatoire parmi les coups optimum)
					Coup = calculerCoupOptimum(Grille, Code)
				case JEU.ALGORITHME.Robot1						// ... et robot (choix déterministe parmi les coups optimum)
					Coup = calculerCoupOptimum1(Grille, Code)
				end
				
				if isequal(Coup, JEU.DAMIER.IMPOSSIBLE)
					JEU.JOUEURS(Code).Etat = JEU.ETAT.Bloque;
					Etat = JEU.JOUEURS(Code).Etat;
					
					voirDamier("Pions", Grille)					// Forçage de l'actualisation du bandeau 
					voirBandeau("JoueurBloqué", Grille, Code)
				else
					JEU.JOUEURS(Code).Etat = JEU.ETAT.Actif;
					Etat = JEU.JOUEURS(Code).Etat;
					
					voirDamier("Pions", Grille)					// Forçage de l'actualisation du bandeau 
					voirBandeau("JoueurActif", Grille, Code, Coup)
				end
			end
			
			if (Etat == JEU.ETAT.Actif)							// Si le joueur courant peut jouer au moins un coup
				Grille = jouerCoup(Grille, Code, Coup)			// Actualisation du damier
				
				[Effectifs, Codes] = calculerEffectifs(Grille)
				for CCode = 1:JEU.Partie.NombreJoueurs
					if (Effectifs(CCode, 1) == 0)				// Recherche des joueurs éliminés
						JEU.JOUEURS(Codes(CCode, 1)).Etat = JEU.ETAT.Elimine;
					end
				end
				
				if (prod(Grille ~= 0))							// Si le damier est rempli > Partie terminée
					JEU.Partie.Etat = JEU.ETAT_PARTIE.COMPLETEE;
				end
				
				sleep(JEU.Partie.PAUSE)
			end
			
			if (Etat == JEU.ETAT.Bloque)						// Si le joueur courant est bloqué ... 
				NombreJoueursBloques = 0;						// ... recherche des autres joueurs bloqués OU éliminés
				for CCode = 1:JEU.Partie.NombreJoueurs
					EEtat = JEU.JOUEURS(CCode).Etat;
					if (EEtat == JEU.ETAT.Bloque)|(EEtat == JEU.ETAT.Elimine)
						NombreJoueursBloques = NombreJoueursBloques + 1;
					end
				end
				//												// Si tous les joueurs sont bloqués ou éliminés > Partie bloquée
				if (NombreJoueursBloques == JEU.Partie.NombreJoueurs)
					JEU.Partie.Etat = JEU.ETAT_PARTIE.BLOQUEE;
				end
				
				sleep(JEU.Partie.PAUSE)
			end
		end														// Fin de la boucle sur le nombre de joueurs sélectionnés
		
		NombreCoups = NombreCoups + 1;							// Incrémentation du compteur anti-boucle infinie
		
		drawnow()
	end															// Fin de la boucle sur les critères d'arrêt de jeu
	
	GrilleS = Grille;
	
	select JEU.Partie.Etat
	case JEU.ETAT_PARTIE.INTERROMPUE then						// Partie interrompue 
		voirBandeau("PartieInterrompue", GrilleS)
		voirDamier("Pions", GrilleS)							// Forçage de l'actualisation du bandeau
	case JEU.ETAT_PARTIE.BLOQUEE then							// Partie bloquee
		voirBandeau("PartieBloquée", GrilleS)
		voirDamier("Pions", GrilleS)							// Forçage de l'actualisation du bandeau 
	case JEU.ETAT_PARTIE.COMPLETEE then							// Partie terminée sans interruption > Affichage du(des) vainqueur(s)
		[Effectifs, Codes] = calculerEffectifs(GrilleS)
		Booleen = (Effectifs == max(Effectifs));
		NombreVainqueurs = sum(Booleen);
		if (NombreVainqueurs == 0)
			CodesVainqueurs = 0;
		else
			CodesVainqueurs = Codes(Booleen);
		end
		
		voirBandeau("PartieTerminée", GrilleS, NombreVainqueurs, CodesVainqueurs)
		voirDamier("Pions", GrilleS)							// Forçage de l'actualisation du bandeau
	end
endfunction
function GrilleS = jouerCoup(Grille, Code, Coup)
	GrilleS = Grille;
	
	if (Grille(Coup(1, 1), Coup(1, 2)) == 0)
		[Score, PionsARetourner] = calculerScore(Grille, Code, Coup)
		
		for n = 1:Score
			GrilleS(PionsARetourner(n, 1), PionsARetourner(n, 2)) = Code;
		end
		
		GrilleS(Coup(1, 1), Coup(1, 2)) = Code;					// Toujours en dernier car calculerScore() ne part que de cases vides
		
		voirDamier("Pions", GrilleS)							// Actualisation de la vue du damier
	end
endfunction
function GrilleS = enleverPions(Grille, Rectangle)
	GrilleS = Grille;
	
	voirBandeau("Décimation", Grille, Rectangle)
	
	GrilleS(Rectangle(1, 1):Rectangle(2, 1), Rectangle(1, 2):Rectangle(2, 2)) = 0;
	
	voirDamier("Pions", GrilleS)
	voirDamier("CasesEffacées")
	voirInterface("DécimationTerminée")
endfunction
function Grille = calculerGrille(Numero)
	global JEU
	
	Grille = zeros(JEU.Partie.Taille, JEU.Partie.Taille);
	
	lC = 3;													//  3ème lettre de l'alphabet
	lD = 4;													//  4ème lettre de l'alphabet
	lE = 5;													//  5ème lettre de l'alphabet
	
	lL = JEU.Partie.Taille/2 - 1;							// 12ème lettre de l'alphabet
	lM = JEU.Partie.Taille/2;								// 13ème lettre de l'alphabet
	//														// La taille du damier doit être toujours paire !!!
	lN = JEU.Partie.Taille/2 + 1;							// 14ème lettre de l'alphabet
	lO = JEU.Partie.Taille/2 + 2;							// 15ème lettre de l'alphabet
	
	lV = JEU.Partie.Taille - 4;								// Quinte-ultième (antepré-antepénultième) lettre de l'alphabet
	lW = JEU.Partie.Taille - 3;								// Quarte-ultième (pré-antepénultième) lettre de l'alphabet
	lX = JEU.Partie.Taille - 2;								// Terce-ultième (antepénultième) lettre de l'alphabet
	
	//lY = JEU.Partie.Taille - 1;							// Pénultième (avant-dernière) lettre de l'alphabet
	//lZ = JEU.Partie.Taille;								// Ultième (dernière) lettre de l'alphabet
	
	select Numero
	case 210 then
		Grille(lM, lM) = JEU.JOUEURS(1).Code;
		Grille(lM, lN) = JEU.JOUEURS(2).Code;
		Grille(lN, lM) = JEU.JOUEURS(2).Code;
		Grille(lN, lN) = JEU.JOUEURS(1).Code;
	case 211 then
		Grille(lM, lM) = JEU.JOUEURS(2).Code;
		Grille(lN, lM) = JEU.JOUEURS(1).Code;
		Grille(lO, lM) = JEU.JOUEURS(2).Code;
		
		Grille(lM, lN) = JEU.JOUEURS(1).Code;
		
		Grille(lO, lN) = JEU.JOUEURS(1).Code;
		
		Grille(lM, lO) = JEU.JOUEURS(2).Code;
		Grille(lN, lO) = JEU.JOUEURS(1).Code;
		Grille(lO, lO) = JEU.JOUEURS(2).Code;
	case 310 then
		Grille(lM, lL) = JEU.JOUEURS(3).Code;
		Grille(lM, lM) = JEU.JOUEURS(1).Code;
		Grille(lM, lN) = JEU.JOUEURS(2).Code;
		Grille(lN, lM) = JEU.JOUEURS(2).Code;
		Grille(lN, lN) = JEU.JOUEURS(1).Code;
		Grille(lN, lO) = JEU.JOUEURS(3).Code;
	case 410 then
		Grille(lL, lL) = JEU.JOUEURS(2).Code;
		Grille(lL, lM) = JEU.JOUEURS(3).Code;
		Grille(lL, lN) = JEU.JOUEURS(4).Code;
		Grille(lL, lO) = JEU.JOUEURS(1).Code;
		
		Grille(lM, lL) = JEU.JOUEURS(3).Code;
		Grille(lM, lM) = JEU.JOUEURS(2).Code;
		Grille(lM, lN) = JEU.JOUEURS(1).Code;
		Grille(lM, lO) = JEU.JOUEURS(4).Code;
		
		Grille(lN, lL) = JEU.JOUEURS(4).Code;
		Grille(lN, lM) = JEU.JOUEURS(1).Code;
		Grille(lN, lN) = JEU.JOUEURS(2).Code;
		Grille(lN, lO) = JEU.JOUEURS(3).Code;
		
		Grille(lO, lL) = JEU.JOUEURS(1).Code;
		Grille(lO, lM) = JEU.JOUEURS(4).Code;
		Grille(lO, lN) = JEU.JOUEURS(3).Code;
		Grille(lO, lO) = JEU.JOUEURS(2).Code;
	case 411 then
		Grille(lL, lL) = JEU.JOUEURS(1).Code;
		Grille(lL, lM) = JEU.JOUEURS(2).Code;
		Grille(lL, lN) = JEU.JOUEURS(3).Code;
		Grille(lM, lN) = JEU.JOUEURS(4).Code;
		Grille(lN, lN) = JEU.JOUEURS(1).Code;
		Grille(lN, lM) = JEU.JOUEURS(2).Code;
		Grille(lN, lL) = JEU.JOUEURS(3).Code;
		Grille(lM, lL) = JEU.JOUEURS(4).Code;
	case 412 then
		Grille(lL, lM) = JEU.JOUEURS(1).Code;
		Grille(lL, lN) = JEU.JOUEURS(2).Code;
		Grille(lM, lL) = JEU.JOUEURS(3).Code;
		Grille(lM, lO) = JEU.JOUEURS(4).Code;
		Grille(lN, lL) = JEU.JOUEURS(4).Code;
		Grille(lN, lO) = JEU.JOUEURS(3).Code;
		Grille(lO, lM) = JEU.JOUEURS(2).Code;
		Grille(lO, lN) = JEU.JOUEURS(1).Code;
	case 420 then
		Grille(lC, lC) = JEU.JOUEURS(1).Code;
		Grille(lC, lD) = JEU.JOUEURS(2).Code;
		Grille(lD, lC) = JEU.JOUEURS(2).Code;
		Grille(lD, lD) = JEU.JOUEURS(1).Code;
		
		Grille(lW, lW) = JEU.JOUEURS(3).Code;
		Grille(lW, lX) = JEU.JOUEURS(4).Code;
		Grille(lX, lW) = JEU.JOUEURS(4).Code;
		Grille(lX, lX) = JEU.JOUEURS(3).Code;
	case 440 then
		Grille(lC, lC) = JEU.JOUEURS(1).Code;
		Grille(lC, lD) = JEU.JOUEURS(2).Code;
		Grille(lD, lC) = JEU.JOUEURS(2).Code;
		Grille(lD, lD) = JEU.JOUEURS(1).Code;
		
		Grille(lW, lC) = JEU.JOUEURS(3).Code;
		Grille(lW, lD) = JEU.JOUEURS(2).Code;
		Grille(lX, lC) = JEU.JOUEURS(2).Code;
		Grille(lX, lD) = JEU.JOUEURS(3).Code;
		
		Grille(lC, lW) = JEU.JOUEURS(1).Code;
		Grille(lC, lX) = JEU.JOUEURS(4).Code;
		Grille(lD, lW) = JEU.JOUEURS(4).Code;
		Grille(lD, lX) = JEU.JOUEURS(1).Code;
		
		Grille(lW, lW) = JEU.JOUEURS(3).Code;
		Grille(lW, lX) = JEU.JOUEURS(4).Code;
		Grille(lX, lW) = JEU.JOUEURS(4).Code;
		Grille(lX, lX) = JEU.JOUEURS(3).Code;
	case 521 then
		Grille(lC, lE) = JEU.JOUEURS(3).Code;
		Grille(lC, lD) = JEU.JOUEURS(4).Code;
		Grille(lC, lC) = JEU.JOUEURS(5).Code;
		Grille(lD, lC) = JEU.JOUEURS(3).Code;
		Grille(lE, lC) = JEU.JOUEURS(4).Code;
		Grille(lE, lD) = JEU.JOUEURS(5).Code;
		
		Grille(lW, lW) = JEU.JOUEURS(1).Code;
		Grille(lW, lX) = JEU.JOUEURS(2).Code;
		Grille(lX, lW) = JEU.JOUEURS(2).Code;
		Grille(lX, lX) = JEU.JOUEURS(1).Code;
	case 621 then
		Grille(lC, lE) = JEU.JOUEURS(1).Code;
		Grille(lC, lD) = JEU.JOUEURS(2).Code;
		Grille(lC, lC) = JEU.JOUEURS(3).Code;
		Grille(lD, lC) = JEU.JOUEURS(1).Code;
		Grille(lE, lC) = JEU.JOUEURS(2).Code;
		Grille(lE, lD) = JEU.JOUEURS(3).Code;
		
		Grille(lX, lV) = JEU.JOUEURS(4).Code;
		Grille(lX, lW) = JEU.JOUEURS(5).Code;
		Grille(lX, lX) = JEU.JOUEURS(6).Code;
		Grille(lW, lX) = JEU.JOUEURS(4).Code;
		Grille(lV, lX) = JEU.JOUEURS(5).Code;
		Grille(lV, lW) = JEU.JOUEURS(6).Code;
	case 630 then
		Grille(lC, lC) = JEU.JOUEURS(1).Code;
		Grille(lC, lD) = JEU.JOUEURS(2).Code;
		Grille(lD, lC) = JEU.JOUEURS(2).Code;
		Grille(lD, lD) = JEU.JOUEURS(1).Code;
		
		Grille(lC, lX) = JEU.JOUEURS(3).Code;
		Grille(lC, lW) = JEU.JOUEURS(4).Code;
		Grille(lD, lX) = JEU.JOUEURS(4).Code;
		Grille(lD, lW) = JEU.JOUEURS(3).Code;
		
		Grille(lX, lC) = JEU.JOUEURS(5).Code;
		Grille(lX, lD) = JEU.JOUEURS(6).Code;
		Grille(lW, lC) = JEU.JOUEURS(6).Code;
		Grille(lW, lD) = JEU.JOUEURS(5).Code;
	case 731 then
		Grille(lC, lE) = JEU.JOUEURS(3).Code;
		Grille(lC, lD) = JEU.JOUEURS(4).Code;
		Grille(lC, lC) = JEU.JOUEURS(5).Code;
		Grille(lD, lC) = JEU.JOUEURS(3).Code;
		Grille(lE, lC) = JEU.JOUEURS(4).Code;
		Grille(lE, lD) = JEU.JOUEURS(5).Code;
		
		Grille(lC, lW) = JEU.JOUEURS(1).Code;
		Grille(lC, lX) = JEU.JOUEURS(2).Code;
		Grille(lD, lW) = JEU.JOUEURS(2).Code;
		Grille(lD, lX) = JEU.JOUEURS(1).Code;
		
		Grille(lW, lC) = JEU.JOUEURS(6).Code;
		Grille(lW, lD) = JEU.JOUEURS(7).Code;
		Grille(lX, lC) = JEU.JOUEURS(7).Code;
		Grille(lX, lD) = JEU.JOUEURS(6).Code;
	case 810 then
		Grille(lL, lL) = JEU.JOUEURS(1).Code;
		Grille(lO, lL) = JEU.JOUEURS(2).Code;
		Grille(lO, lO) = JEU.JOUEURS(3).Code;
		Grille(lL, lO) = JEU.JOUEURS(4).Code;
		
		Grille(lN, lM) = JEU.JOUEURS(1).Code;
		Grille(lN, lN) = JEU.JOUEURS(2).Code;
		Grille(lM, lN) = JEU.JOUEURS(3).Code;
		Grille(lM, lM) = JEU.JOUEURS(4).Code;
		
		Grille(lM, lL) = JEU.JOUEURS(5).Code;
		Grille(lN, lL) = JEU.JOUEURS(6).Code;
		Grille(lO, lM) = JEU.JOUEURS(7).Code;
		Grille(lO, lN) = JEU.JOUEURS(8).Code;
		
		Grille(lN, lO) = JEU.JOUEURS(5).Code;
		Grille(lM, lO) = JEU.JOUEURS(6).Code;
		Grille(lL, lN) = JEU.JOUEURS(7).Code;
		Grille(lL, lM) = JEU.JOUEURS(8).Code;
	case 840 then
		Grille(lC, lC) = JEU.JOUEURS(1).Code;
		Grille(lC, lD) = JEU.JOUEURS(2).Code;
		Grille(lD, lC) = JEU.JOUEURS(2).Code;
		Grille(lD, lD) = JEU.JOUEURS(1).Code;
		
		Grille(lW, lC) = JEU.JOUEURS(3).Code;
		Grille(lW, lD) = JEU.JOUEURS(4).Code;
		Grille(lX, lC) = JEU.JOUEURS(4).Code;
		Grille(lX, lD) = JEU.JOUEURS(3).Code;
		
		Grille(lC, lW) = JEU.JOUEURS(5).Code;
		Grille(lC, lX) = JEU.JOUEURS(6).Code;
		Grille(lD, lW) = JEU.JOUEURS(6).Code;
		Grille(lD, lX) = JEU.JOUEURS(5).Code;
		
		Grille(lW, lW) = JEU.JOUEURS(7).Code;
		Grille(lW, lX) = JEU.JOUEURS(8).Code;
		Grille(lX, lW) = JEU.JOUEURS(8).Code;
		Grille(lX, lX) = JEU.JOUEURS(7).Code;
	end
endfunction

function Coup = calculerCoupOptimum(Grille, Code)
	global JEU
	
	ListeCoups = calculerCoups(Grille, Code);
	
	if (ListeCoups == [])
		Coup = JEU.DAMIER.IMPOSSIBLE;
	else
		BordsMax = max(ListeCoups(:, 4));
		IndicesBordsMax = find(ListeCoups(:, 4) == BordsMax);
		ListeCoupsBordsMax = ListeCoups(IndicesBordsMax, 1:3);
		
		ScoreMax = max(ListeCoupsBordsMax(:, 3));
		IndicesScoreMax = find(ListeCoupsBordsMax(:, 3) == ScoreMax);
		ListeCoupsScoreMax = ListeCoupsBordsMax(IndicesScoreMax, 1:2);
		
		IndiceOptimum = 1 + fix(size(ListeCoupsScoreMax, 1)*rand());
		Coup = ListeCoupsScoreMax(IndiceOptimum, :);
	end
endfunction
function Coup = calculerCoupOptimum1(Grille, Code)
	global JEU
	
	ListeCoups = calculerCoups(Grille, Code);
	
	if (ListeCoups == [])
		Coup = JEU.DAMIER.IMPOSSIBLE;
	else
		BordsMax = max(ListeCoups(:, 4));
		IndicesBordsMax = find(ListeCoups(:, 4) == BordsMax);
		ListeCoupsBordsMax = ListeCoups(IndicesBordsMax, 1:3);
		
		ScoreMax = max(ListeCoupsBordsMax(:, 3));
		IndicesScoreMax = find(ListeCoupsBordsMax(:, 3) == ScoreMax);
		ListeCoupsScoreMax = ListeCoupsBordsMax(IndicesScoreMax, 1:2);
		
		Coup = ListeCoupsScoreMax(1, :);
	end
endfunction
function Coup = jouerHumain(Grille, Code)
	global JEU
	
	ListeCoups = calculerCoups(Grille, Code);
	
	if (ListeCoups == [])
		Coup = JEU.DAMIER.IMPOSSIBLE;
	else
		AttendreSelection = %T;
		
		while (AttendreSelection)
			Coup = JEU.DAMIER.IMPOSSIBLE;
			
			Reponse = xgetmouse();
			x = Reponse(1);
			y = Reponse(2);
			Action = Reponse(3);
			
			if (Action == JEU.DAMIER.Selection)
				Coup = [min(1 + floor(x), JEU.Partie.Taille), ...
							min(1 + floor(y), JEU.Partie.Taille)];
			end
			if (Coup ~= JEU.DAMIER.IMPOSSIBLE)
				CaseVide = (Grille(Coup(1, 1), Coup(1, 2)) == 0);
				CaseJouable = sum((Coup(1, 1) == ListeCoups(:, 1)).*(Coup(1, 2) == ListeCoups(:, 2)));
				
				if (CaseVide)&(CaseJouable)
					AttendreSelection = %F;
				end
			end
		end
	end
endfunction
function [Rectangle, SelectionEffectuee] = selectionnerRectangle(Mode)
	global JEU
	
	select Mode
	case "Humain" 												// Mode automatique : l'usager sélectionne
		ContinuerSelection = %T;
		NombreSelections = 0;
		Rectangle = JEU.DAMIER.IMPOSSIBLE;
		
		while (ContinuerSelection)
			[Action, x, y] = xclick();							// Force pause dans l'exécution de la partie
			
			NombreSelections = NombreSelections + 1;
			
			select Action
			case JEU.DAMIER.Selection then
				if (NombreSelections == 1)
					Liste = min(1 + floor([x, y]), JEU.Partie.Taille);
					
					Rectangle = [Liste; Liste];
				end
				if (NombreSelections > 1)
					Liste = [Liste; min(1 + floor([x, y]), JEU.Partie.Taille)];
					
					i = Liste(($ - 1):$, 1);
					j = Liste(($ - 1):$, 2);
					
					Rectangle = [[min(i), min(j)]; [max(i), max(j)]];
				end
				
				voirDamier("CasesSélectionnées", Rectangle, color("orange"))
			case JEU.DAMIER.Confirmation then
				ContinuerSelection = %F;
				SelectionEffectuee = %T;
			case JEU.DAMIER.Annulation then			
				ContinuerSelection = %F;
				SelectionEffectuee = %F;
			end
		end
	case "Aléatoire"											// Un joueur humain : décimation aléatoire
		i = fix(1 + JEU.Partie.Taille*rand(2, 1));
		j = fix(1 + JEU.Partie.Taille*rand(2, 1));
		
		Rectangle = [[min(i), min(j)]; [max(i), max(j)]];
		SelectionEffectuee = %T;
		
		voirDamier("CasesSélectionnées", Rectangle, color("red"))
	end
endfunction

function ListeCoups = calculerCoups(Grille, Code)
	global JEU
	
	ListeCoups = [];
	
	for i = 1:JEU.Partie.Taille
		for j = 1:JEU.Partie.Taille
			[Score, PionsARetourner] = calculerScore(Grille, Code, [i, j])
			if (Score ~= 0)
				Bords = (i == 1)|(i == JEU.Partie.Taille) + (j == 1)|(j == JEU.Partie.Taille);
				ListeCoups = [ListeCoups; [i, j, Score, Bords]];
			end
		end
	end
endfunction
function [Score, PionsARetourner] = calculerScore(Grille, Code, Coup)
	global JEU
	
	l = Coup(1, 1);
	c = Coup(1, 2);
	
	PionsARetourner = [];
	
	if (Grille(l, c) == 0)
		for d = 1:JEU.DAMIER.DIRECTIONS_NOMBRE					// Exploration des directions de voisinage possibles.
			dx = JEU.DAMIER.DIRECTIONS(d, 1);
			dy = JEU.DAMIER.DIRECTIONS(d, 2);
			
			DirectionExplorable = %F;
			PionsListe = [];
			
			ll = l + dx;
			cc = c + dy;
			CaseInterieure = ((ll >= 1)&(ll <= JEU.Partie.Taille)&(cc >= 1)&(cc <= JEU.Partie.Taille));
			if (CaseInterieure)									// Il y a une case adjacente sur le damier (dans cette direction)
				//												// Le pion adjacent appartient à un autre joueur.
				if (Grille(ll, cc) ~= 0)&(Grille(ll, cc) ~= Code)
					DirectionExplorable = %T;
				end
			end
			while (DirectionExplorable)
				PionsListe = [PionsListe; [ll, cc]];
				
				ll = ll + dx;
				cc = cc + dy;
				CaseInterieure = ((ll >= 1)&(ll <= JEU.Partie.Taille)&(cc >= 1)&(cc <= JEU.Partie.Taille));
				if (~CaseInterieure)							// Il n'y a plus de case adjacente dans le damier.
					DirectionExplorable = %F;
					break
				end
				if (Grille(ll, cc) == 0)						// La case adjacente est vide.
					DirectionExplorable = %F;
					break
				end
				if (Grille(ll, cc) == Code)						// Il y a une possiblité d'encerclement.
					DirectionExplorable = %F;
					PionsARetourner = [PionsARetourner; PionsListe];
					break
				end
			end
		end	
	end
	
	Score = size(PionsARetourner, 1);
endfunction
function [Effectifs, Codes] = calculerEffectifs(Grille)
	global JEU
	
	Effectifs = [];
	
	for Code = 1:JEU.Partie.NombreJoueurs
		Effectifs = [Effectifs; sum(Grille == Code)];
	end
	
	[Effectifs, Codes] = gsort(Effectifs);
endfunction

////////////////////////////////////////////////////////////////////////////////
//
//	FONCTIONS VUES
  //
  //	Interface	Gestion des menus de sélection et des boutons d'action
  //	Damier		Gestion de l'affichage du damier, des cases et des pions 
  //	Bandeau		Messages d'information dans le bandeau
//
////////////////////////////////////////////////////////////////////////////////

function voirInterface(Action)
	global JEU
	
	select Action
	case "Création" then
		// Création de l'interface
			interfaceJeu = figure('figure_position', JEU.INTERFACE.Position)
			interfaceJeu.Tag = JEU.INTERFACE.Nom;
			
			interfaceJeu.figure_size = JEU.INTERFACE.Taille;
			interfaceJeu.auto_resize = 'on';
			interfaceJeu.figure_name = JEU.INTERFACE.Titre;
			interfaceJeu.backgroundcolor = JEU.INTERFACE.CouleurRVB;
			
			delmenu(interfaceJeu.figure_id, gettext('File'))
			delmenu(interfaceJeu.figure_id, gettext('?'))
			delmenu(interfaceJeu.figure_id, gettext('Tools'))
			toolbar(interfaceJeu.figure_id, 'off')
		//
		// Création du damier
			voirDamier("Création", JEU.Partie.Taille)
			voirDamier("Initialisation", JEU.Partie.Taille)
		//
		// Création du bandeau d'information
			voirBandeau("Création", [])
			voirBandeau("Initialisation", [])
		//
		
		// Menu de sélection de la grille de départ
			menu1 = uimenu(interfaceJeu, "Tag", JEU.MENUS.Nom11, "Label", "Grilles");
			
			uimenu(menu1, "Label", "2 joueurs 1 noyau",       "Callback", "selectionnerGrille(210)")
			uimenu(menu1, "Label", "2 joueurs 1 noyau creux", "Callback", "selectionnerGrille(211)")
			
			uimenu(menu1, "Label", "3 joueurs 1 noyau",       "Callback", "selectionnerGrille(310)")
			
			uimenu(menu1, "Label", "4 joueurs 1 noyau",       "Callback", "selectionnerGrille(410)")
			uimenu(menu1, "Label", "4 joueurs 1 noyau creux", "Callback", "selectionnerGrille(411)")
			uimenu(menu1, "Label", "4 joueurs 1 noyau creux", "Callback", "selectionnerGrille(412)")
			uimenu(menu1, "Label", "4 joueurs 2 noyaux",      "Callback", "selectionnerGrille(420)")
			uimenu(menu1, "Label", "4 joueurs 4 noyaux",      "Callback", "selectionnerGrille(440)")
			
			uimenu(menu1, "Label", "5 joueurs 2 noyaux",      "Callback", "selectionnerGrille(521)")
			
			uimenu(menu1, "Label", "6 joueurs 2 noyaux creux","Callback", "selectionnerGrille(621)")
			uimenu(menu1, "Label", "6 joueurs 3 noyaux",      "Callback", "selectionnerGrille(630)")
			
			uimenu(menu1, "Label", "7 joueurs 3 noyaux",      "Callback", "selectionnerGrille(731)")
			
			uimenu(menu1, "Label", "8 joueurs 1 noyau",       "Callback", "selectionnerGrille(810)")
			uimenu(menu1, "Label", "8 joueurs 4 noyaux",      "Callback", "selectionnerGrille(840)")
		//
		// Menu de sélection de la taille de la grille
			menu2 = uimenu(interfaceJeu, "Tag", JEU.MENUS.Nom12, "Label", "Tailles");
			
			for taille = 8:2:20
				uimenu(menu2,	"Label",	string(taille) + " x " + string(taille),	...
								"Callback",	"selectionnerTaille(" + string(taille) + ")")
			end
		//
		// Menu de sélection du joueur humain		// Pas plus d'un niveau d'arborescence pour les pointeur
			menu3 = uimenu(interfaceJeu, "Tag", JEU.MENUS.Nom13, "Label", "Joueur humain");
			
			uimenu(menu3, "Label", "Aucun", "Callback", "selectionnerHumain(0)")
			uimenu(menu3, "Label", "")
			
			for code = 1:JEU.JOUEURS_NOMBRE
				uimenu(menu3,	"Tag",		JEU.MENUS.Nom13Choix(code),					...
								"Label", 	JEU.JOUEURS(code).Nom,						...
								"Callback", "selectionnerHumain(" + string(code) + ")")
			end					
		//
		
		// Commande de lancement de la partie
			bouton15 = uicontrol(interfaceJeu, "style", "pushbutton");
			bouton15.Tag = JEU.BOUTONS.Nom15;
			bouton15.Units = "normalized";
			bouton15.Position = JEU.BOUTONS.Position15;
			bouton15.FontSize = JEU.BOUTONS.TaillePolice;
			bouton15.String = "Jouer";
			bouton15.BackgroundColor = JEU.BOUTONS.CouleurRVB;
			bouton15.Callback = "controlerPartie()";
			bouton15.Relief = "raised";
		//
		// Commande d'affichage des coups jouables
			bouton25 = uicontrol(interfaceJeu, "style", "pushbutton");
			bouton25.Tag = JEU.BOUTONS.Nom25;
			bouton25.Units = "normalized";
			bouton25.Position = JEU.BOUTONS.Position25;
			bouton25.FontSize = JEU.BOUTONS.TaillePolice;
			bouton25.String = "Suggèrer";
			bouton25.BackgroundColor = JEU.BOUTONS.CouleurRVB;
			bouton25.Callback = "controlerAffichageAide()";
			bouton25.Relief = "raised";
			bouton25.Visible = "off";
		//
		// Commande de décimation (aléatoire si un joueur humain, sélective sinon)
			bouton35 = uicontrol(interfaceJeu, "style", "pushbutton");
			bouton35.Tag = JEU.BOUTONS.Nom35;
			bouton35.Units = "normalized";
			bouton35.Position = JEU.BOUTONS.Position35;
			bouton35.FontSize = JEU.BOUTONS.TaillePolice;
			bouton35.String = "Décimer";
			bouton35.BackgroundColor = JEU.BOUTONS.CouleurRVB;
			bouton35.Callback = "controlerDecimationPions()";
			bouton35.Relief = "raised";
			bouton35.Visible = "off";
		//
		// Commande d'interruption de la partie
			bouton45 = uicontrol(interfaceJeu, "style", "pushbutton");
			bouton45.Tag = JEU.BOUTONS.Nom45;
			bouton45.Units = "normalized";
			bouton45.Position = JEU.BOUTONS.Position45;
			bouton45.FontSize = JEU.BOUTONS.TaillePolice;
			bouton45.String = "Arrêter partie";
			bouton45.BackgroundColor = JEU.BOUTONS.CouleurRVB;
			bouton45.Callback = "arreterPartie()";
			bouton45.Relief = "raised";
			bouton45.Visible = "off";
		//
		// Commande de sortie du jeu
			bouton55 = uicontrol(interfaceJeu, "style", "pushbutton");
			bouton55.Tag = JEU.BOUTONS.Nom55;
			bouton55.Units = "normalized";
			bouton55.Position = JEU.BOUTONS.Position55;
			bouton55.FontSize = JEU.BOUTONS.TaillePolice;
			bouton55.String = "Sortir";
			bouton55.BackgroundColor = JEU.BOUTONS.CouleurRVB;
			bouton55.Callback = "sortir()";
			bouton55.Relief = "raised";
			bouton55.Visible = "on";
		//
	case "GrilleSélectionnée" then
		for code = 1:JEU.Partie.NombreJoueurs
			set(get(JEU.MENUS.Nom13Choix(code, 1)), "Visible", "on")
		end
		for code = (JEU.Partie.NombreJoueurs + 1):JEU.JOUEURS_NOMBRE
			set(get(JEU.MENUS.Nom13Choix(code, 1)), "Visible", "off")
		end
	case "PartieRéinitialisée" then
		set(get(JEU.MENUS.Nom11),	"Visible",	"on")
		set(get(JEU.MENUS.Nom12),	"Visible",	"on")
		set(get(JEU.MENUS.Nom13),	"Visible",	"on")
		
		set(get(JEU.BOUTONS.Nom15),	"String",	"Jouer",	"Relief",	"raised",	"Enable",	"on")
		set(get(JEU.BOUTONS.Nom25),	"Visible",	"off")
		set(get(JEU.BOUTONS.Nom35),	"Visible",	"off")
		set(get(JEU.BOUTONS.Nom45),	"Visible",	"off")
		set(get(JEU.BOUTONS.Nom55),	"Visible",	"on")
	case "PartieEnCours" then
		set(get(JEU.MENUS.Nom11),	"Visible",	"off")
		set(get(JEU.MENUS.Nom12),	"Visible",	"off")
		set(get(JEU.MENUS.Nom13),	"Visible",	"off")
		
		set(get(JEU.BOUTONS.Nom15),	"Relief",	"flat",		"Enable",	"off")
		set(get(JEU.BOUTONS.Nom25),	"Visible",	"on")
		set(get(JEU.BOUTONS.Nom35),	"Visible",	"on")
		set(get(JEU.BOUTONS.Nom45),	"Visible",	"on")
		set(get(JEU.BOUTONS.Nom55),	"Visible",	"off")
	case "PartieTerminée" then
		set(get(JEU.MENUS.Nom11),	"Visible",	"on")
		set(get(JEU.MENUS.Nom12),	"Visible",	"on")
		set(get(JEU.MENUS.Nom13),	"Visible",	"on")
		
		set(get(JEU.BOUTONS.Nom15),	"String",	"Effacer",	"Relief",	"raised",	"Enable",	"on")
		set(get(JEU.BOUTONS.Nom25),	"Visible",	"off")
		set(get(JEU.BOUTONS.Nom35),	"Visible",	"off")
		set(get(JEU.BOUTONS.Nom45),	"Visible",	"off")
		set(get(JEU.BOUTONS.Nom55),	"Visible",	"on")
	case "CasesJouables" then
		set(get(JEU.BOUTONS.Nom25),	"Relief",	"flat")
	case "CasesJouablesMasquées" then
		set(get(JEU.BOUTONS.Nom25),	"Relief",	"raised")
	case "Décimation" then
		set(get(JEU.BOUTONS.Nom35),	"Enable",	"off",		"Relief",	"flat")
	case "DécimationTerminée" then
		set(get(JEU.BOUTONS.Nom35),	"Enable",	"on",		"Relief",	"raised")
	end
	
	drawnow()
endfunction
function voirDamier(Action, varargin)
	global JEU
	
	select Action
	case "Création" then
		Taille = varargin(1)
		
		damier = newaxes(get(JEU.INTERFACE.Nom));
		damier.tag = "Damier";
		
		damier.axes_bounds = JEU.DAMIER.Position;
		damier.margins = JEU.DAMIER.Marges;
		
		damier.axes_visible = "off";
		damier.x_location = "top";
		damier.y_location = "left";
		damier.box = "off";
		damier.isoview = "on";
		damier.data_bounds = [0, 0; Taille, Taille];
	case "Initialisation" then
		Taille = varargin(1)
		
		for i = 1:Taille
			for j = 1:Taille
				JEU.Cases(i, j) = "CaseL" + string(i) + "C" + string(j);	
				xfpoly(i - 1 + JEU.CASE.X, j - 1 + JEU.CASE.Y, JEU.CASE.CouleurCode)
				set(gce(), "Tag", JEU.Cases(i, j))
				
				JEU.Cadres(i, j) = "CadreL" + string(i) + "C" + string(j);	
				xpoly(i - 1 + JEU.CASE.X, j - 1 + JEU.CASE.Y, "lines")
				set(gce(), "Tag", JEU.Cadres(i, j))	
				
				JEU.Pions(i, j) = "PionL" + string(i) + "C" + string(j);
				xfpoly(i - 1, j - 1, JEU.CASE.CouleurCode)
				set(gce(), "Tag", JEU.Pions(i, j))	
			end
		end
	case "Destruction" then
		Taille = varargin(1)
		
		objet = get("Damier")
		delete(objet.children)
		objet.data_bounds = [0, 0; Taille, Taille];			// Actualisation des dimensions du damier
		
		JEU.Cases = "";
		JEU.Cadres = "";
		JEU.Pions = "";	
	case "Pions" then
		Grille = varargin(1)
		
		for i = 1:JEU.Partie.Taille
			for j = 1:JEU.Partie.Taille
				if (Grille(i, j) == 0)
					set(get(JEU.Pions(i, j)), "Data", [i - 1, j - 1], "Background", JEU.CASE.CouleurCode)
				end
				if (Grille(i, j) ~=0)
					set(get(JEU.Pions(i, j)),											...
						"Data",					[i - 1 + JEU.JOUEURS(Grille(i, j)).X,	...
												 j - 1 + JEU.JOUEURS(Grille(i, j)).Y],	...
						"Background",			JEU.JOUEURS(Grille(i, j)).CouleurCode);
				end
			end
		end
	case "CasesJouables" then
		ListeCasesJouables = varargin(1)
		CodeCouleur = varargin(2)
		
		for n = 1:size(ListeCasesJouables, 1)
			l = ListeCasesJouables(n, 1);
			c = ListeCasesJouables(n, 2);
			
			set(get(JEU.Pions(l, c)), "Data", [l - 1 + JEU.CASE_JOUABLE.X, c - 1 + JEU.CASE_JOUABLE.Y], "Background", CodeCouleur)
		end		
	case "CasesSélectionnées" then
		Rectangle = varargin(1)
		CodeCouleur = varargin(2)
		
		for i = 1:JEU.Partie.Taille
			for j = 1:JEU.Partie.Taille
				set(get(JEU.Cases(i, j)), "Background", JEU.CASE.CouleurCode)
			end
		end
		for i = Rectangle(1, 1):Rectangle(2, 1)
			for j = Rectangle(1, 2):Rectangle(2, 2)
				set(get(JEU.Cases(i, j)), "Background", CodeCouleur)
			end
		end
	case "CasesEffacées" then
		for i = 1:JEU.Partie.Taille
			for j = 1:JEU.Partie.Taille
				set(get(JEU.Cases(i, j)), "Background", JEU.CASE.CouleurCode)
			end
		end
	end
	
	drawnow()
endfunction
function voirCasesJouables(Grille, Code)
	if (Code ~= 0)
		ListeCasesJouables = calculerCoups(Grille, Code);
		
		voirDamier("CasesJouables", ListeCasesJouables, JEU.JOUEURS(Code).CouleurCode)
	end
endfunction
function voirBandeau(Action, Grille, varargin)
	global JEU
	
	if (Action ~= "Création")&(Action ~= "Initialisation")
		ligne1 = string(JEU.Partie.Taille) + " x " + string(JEU.Partie.Taille) + " cases | ";
		ligne1 = ligne1 + string(JEU.Partie.NombreJoueurs) + " joueurs | ";
		if (JEU.Partie.CodeHumain == 0)
			ligne1 = ligne1 + "Mode automatique";
		else
			ligne1 = ligne1 + "Humain joue " + JEU.JOUEURS(JEU.Partie.CodeHumain).Nom;
		end
		
		ligne2 = "";
		[Effectifs, Codes] = calculerEffectifs(Grille)
		for code = 1:JEU.Partie.NombreJoueurs
			if (Effectifs(code, 1) ~= 0)
				Nom = part(JEU.JOUEURS(Codes(code, 1)).Nom, 1:3);
				
				ligne2 = ligne2 + Nom + " (" + string(Effectifs(code, 1)) + ") ";
			end
		end
	end
	
	select Action
	case "Création" then
		bandeau = uicontrol(get(JEU.INTERFACE.Nom), "Style", "text");
		bandeau.Tag = JEU.BANDEAU.Nom;
		bandeau.Position = JEU.BANDEAU.Position;
		bandeau.FontSize = JEU.BANDEAU.TaillePolice;
		bandeau.FontWeight = "bold";
		bandeau.HorizontalAlignment = "left";
		bandeau.String = "";
		bandeau.BackgroundColor = JEU.BANDEAU.CouleurRVB;
	case "Initialisation" then
		ligne1 = "Taille de 8 à 20 | 2 à 8 joueurs | 1 joueur humain maximum";
		ligne2 = "";
		ligne3 = "";
	case "Sélection" then
		ligne3 = "Sélectionnez grille | taille | joueur humain";
	case "JoueurActif" then
		Code = varargin(1);
		Coup = varargin(2);
		
		ligne3 = JEU.JOUEURS(Code).Nom;
		ligne3 = ligne3 + " joue en case (";
		ligne3 = ligne3 + string(Coup(1, 1)) + ", " + string(Coup(1, 2)) + ")";
	case "JoueurBloqué" then
		Code = varargin(1);
		
		ligne3 = JEU.JOUEURS(Code).Nom;
		ligne3 = ligne3 + " bloqué";
	case "Décimation" then
		Rectangle = varargin(1);
		
		ligne3 = "Eradication dans le rectangle [";
		ligne3 = ligne3 + string(Rectangle(1, 1)) + ", " + string(Rectangle(1, 2)) + "; ";
		ligne3 = ligne3 + string(Rectangle(2, 1)) + ", " + string(Rectangle(2, 2)) + "]";
	case "PartieInterrompue" then
		ligne3 = "Interruption de la partie";
	case "PartieBloquée" then
		ligne3 = "Partie bloquée : tous les joueurs restants sont bloqués";
	case "PartieTerminée" then
		NombreVainqueurs = varargin(1);
		CodesVainqueurs = varargin(2);
		
		select NombreVainqueurs
		case 0 then
			ligne3 = "Partie terminée sans vainqueur ...";
		case 1 then
			ligne3 = "Victoire du joueur " + JEU.JOUEURS(CodesVainqueurs).Nom;
		else
			ligne3 = "Joueurs " 
			for code = 1:NombreVainqueurs
				ligne3 = ligne3 + JEU.JOUEURS(CodesVainqueurs(code, 1)).Nom + " ";
			end
			ligne3 = ligne3 + "ex-aequo";
		end
	else
		ligne1 = "Erreur d''argument d''entrée :";
		ligne2 = "action " + action;
		ligne3 = "non traitée par la fonction voirBandeau()";
	end
	
	if (Action ~= "Création")
		//mprintf(ligne1 + "\n" + ligne2 + "\n" + ligne3 + "\n\n")
		set(get(JEU.BANDEAU.Nom), "String", "$\textbf{" + ligne1 + "}\\ \textbf{" + ligne2 + "}\\ \textbf{" + ligne3 + "}$")
	end
	
	drawnow()
endfunction

////////////////////////////////////////////////////////////////////////////////
//
//	FONCTIONS CONTROLEURS
  //
  //	Menu	Sélection d'une grille de départ
  //	Menu	Sélection de la taille de la grille
  //	Menu	Sélection du joueur humain
  //	
  //	Bouton	Nouvelle partie
  //	Bouton	Affichage des cases jouables
  //	Bouton	Elimination de pions pour prolonger la partie
  //	Bouton	Arrêt de la partie en cours
  //	Bouton	Sortie du jeu
//
////////////////////////////////////////////////////////////////////////////////

function selectionnerGrille(Numero)
	global JEU
	
	JEU.Partie.NumeroGrille = Numero;
	JEU.Partie.NombreJoueurs = fix(Numero/100);
	JEU.Partie.GrilleDebut = calculerGrille(Numero);
	
	for code = 1:JEU.Partie.NombreJoueurs
		JEU.JOUEURS(code).Etat = JEU.ETAT.Actif;
		if (code ~= JEU.Partie.CodeHumain)
			JEU.JOUEURS(code).Algorithme = JEU.ALGORITHME.Robot;
		end
	end
	for code = (JEU.Partie.NombreJoueurs + 1):JEU.JOUEURS_NOMBRE
		JEU.JOUEURS(code).Etat = JEU.ETAT.Inactif;
		JEU.JOUEURS(code).Algorithme = JEU.ALGORITHME.Robot;
	end
	
	voirInterface("GrilleSélectionnée")
	voirDamier("Pions", JEU.Partie.GrilleDebut)
	voirBandeau("Sélection", JEU.Partie.GrilleDebut)
endfunction
function selectionnerTaille(Taille)
	global JEU
	
	JEU.Partie.Taille = Taille;
	JEU.Partie.GrilleDebut = calculerGrille(JEU.Partie.NumeroGrille)
	
	voirDamier("Destruction", JEU.Partie.Taille)
	voirDamier("Initialisation", JEU.Partie.Taille)
	voirDamier("Pions", JEU.Partie.GrilleDebut)
	voirBandeau("Sélection", JEU.Partie.GrilleDebut)
endfunction
function selectionnerHumain(CodeHumain)
	global JEU
	
	JEU.Partie.CodeHumain = CodeHumain;
	
	for Code = 1:JEU.Partie.NombreJoueurs
		JEU.JOUEURS(Code).Algorithme = JEU.ALGORITHME.Robot;
	end
	if (CodeHumain ~= 0)
		JEU.JOUEURS(CodeHumain).Algorithme = JEU.ALGORITHME.Humain;
	end
	
	voirDamier("Pions", JEU.Partie.GrilleDebut)
	voirBandeau("Sélection", JEU.Partie.GrilleDebut)
endfunction

function controlerPartie()
	global JEU
	
	select JEU.Partie.Etat
	case JEU.ETAT_PARTIE.REINITIALISABLE then					// Effacement de la partie précédente
		JEU.Partie.GrilleDebut = calculerGrille(JEU.Partie.NumeroGrille)
		
		voirDamier("Pions", JEU.Partie.GrilleDebut)
		voirInterface("PartieRéinitialisée")
		
		JEU.Partie.Etat = JEU.ETAT_PARTIE.ACTIVABLE;		
	case JEU.ETAT_PARTIE.ACTIVABLE then							// Lancement d'une nouvelle partie
		JEU.Partie.GrilleDebut = calculerGrille(JEU.Partie.NumeroGrille)
		
		voirDamier("Pions", JEU.Partie.GrilleDebut)
		voirInterface("PartieEnCours")
		
		JEU.Partie.GrilleFin = jouerPartie(JEU.Partie.GrilleDebut)
		
		voirInterface("PartieTerminée")							// Fin de la nouvelle la partie
		
		JEU.Partie.Etat = JEU.ETAT_PARTIE.REINITIALISABLE;
	end
endfunction
function controlerAffichageAide()
	global JEU
	
	select JEU.Partie.AffichageAide
	case 0 then									// Pas d'affichage = > Affichage pour le joueur humain
		JEU.Partie.AffichageAide = 1;
		
		if (JEU.Partie.CodeHumain == 0)
			controlerAffichageAide()
		end
		
		voirInterface("CasesJouables")
	case 1 then									// Affichage pour le joueur humain => Affichage pour tous les joueurs
		JEU.Partie.AffichageAide = 2;
		
		voirInterface("CasesJouables")
	case 2 then									// Affichage pour tous les joueurs => Pas d'affichage
		JEU.Partie.AffichageAide = 0;
		
		voirInterface("CasesJouablesMasquées")
	end
endfunction
function controlerDecimationPions()
	global JEU
	
	select JEU.Partie.Decimation
	case 0 then									// Sélection des pions à décimer
		voirInterface("Décimation")
		
		if (JEU.Partie.CodeHumain == 0)
			[Rectangle, SelectionEffectuee] = selectionnerRectangle("Humain")
		else
			[Rectangle, SelectionEffectuee] = selectionnerRectangle("Aléatoire")
		end
		
		if (SelectionEffectuee)
			JEU.Selection = Rectangle;
			
			voirDamier("CasesSélectionnées", JEU.Selection, color("red"))
			
			JEU.Partie.Decimation = 1;
		else
			voirDamier("CasesEffacées")
			voirInterface("DécimationTerminée")
		
			JEU.Partie.Decimation = 0;
		end
	case 1 then									// Décimation des pions sélectionnés
		voirInterface("DécimationTerminée")
		
		JEU.Partie.Decimation = 0;
	end
endfunction
function arreterPartie()
	global JEU
	
	JEU.Partie.Etat = JEU.ETAT_PARTIE.INTERROMPUE;
endfunction
function sortir()
	xdel(winsid())
endfunction

///////////////////////////////////////////////////////////////////////////////
//
//	PROGRAMME PRINCIPAL
//
///////////////////////////////////////////////////////////////////////////////

affecterParametres()
voirInterface("Création")
selectionnerGrille(JEU.Partie.NumeroGrille)